﻿using System;
using System.ServiceModel;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using TaskClient.TaskService.Proxy;

namespace TaskClient
{
	public partial class TaskForm : Form
	{
		private TaskManagerServiceClient _taskMgrProxy = new TaskManagerServiceClient();
		private string _name;

		public TaskForm( string name)
		{
			InitializeComponent();

			_name = name;
		}

		private void _btnSubmitTask_Click(object sender, EventArgs e)
		{
			string assignedTo = _txtAssignToName.Text;
			int newTaskNum = _taskMgrProxy.AddTask(_rtbNewTaskDescription.Text, assignedTo);

			_statusLabel.Text = string.Format("Task number {0} was submitted", newTaskNum);

			_rtbNewTaskDescription.Text = string.Empty;
			_txtAssignToName.Text = string.Empty;

			_rtbNewTaskDescription.Focus();

			GetTaskNumbersFromService();
		}

		private void _taskList_SelectedIndexChanged(object sender, EventArgs e)
		{
			int taskNum = int.Parse(_taskList.SelectedItem.ToString());
			string taskDesc;
			try
			{
				taskDesc = _taskMgrProxy.GetTaskDescription(taskNum);
			}

			catch (FaultException<FaultInfo> fault)
			{
				MessageBox.Show("There was a problem retrieving task description from the service.\n" + fault.Detail.Reason);
				taskDesc = string.Empty;
			}

			_checkBoxCompleted.Checked = _taskMgrProxy.IsTaskCompleted(taskNum);
			_rtbTaskDescription.Text = taskDesc;
		}

		private void _checkBoxCompleted_CheckedChanged(object sender, EventArgs e)
		{
			if (_checkBoxCompleted.Checked)
			{
				int selIndex = _taskList.SelectedIndex;
				int taskNum = int.Parse(_taskList.SelectedItem.ToString());

				try
				{
					_taskMgrProxy.MarkTaskCompleted(taskNum);
				}

				catch (FaultException<FaultInfo> fi)
				{
					MessageBox.Show(
						"There was a problem mark a task completed with the service.\n" +
							fi.Detail.Reason);
				}
			}
		}

		private void TaskForm_Load(object sender, EventArgs e)
		{
			this.Text = "Task Manager: " + _name;

			GetTaskNumbersFromService();
		}

		private void GetTaskNumbersFromService( )
		{
			// First clear the list...
			_taskList.Items.Clear();

			// And then load it:
			int[] taskNums = _taskMgrProxy.GetTasksByAssignedName(_name);
			foreach( int tn in taskNums)
				_taskList.Items.Add(tn);
		}

		private void _btnDeleteTask_Click(object sender, EventArgs e)
		{
			int taskNum = int.Parse(_taskList.SelectedItem.ToString());

			try
			{
				_taskMgrProxy.DeleteTask(taskNum);

				GetTaskNumbersFromService();

				if (_taskList.Items.Count > 0)
				{
					_taskList.SelectedIndex = 0;
				}
				else
				{
					_rtbTaskDescription.Text = string.Empty;
					_checkBoxCompleted.Checked = false;
				}
			}

			catch (FaultException<FaultInfo> fi)
			{
				MessageBox.Show(
					"There was a problem deleting a task with the service.\n" +
						fi.Detail.Reason + "\nPlease try again later.");
			}
		}
	}
}
