﻿Imports System.ServiceModel
Imports TaskClient.TaskService.Proxy

Public Class TaskForm
    Private _taskMgrProxy As TaskManagerServiceClient = New TaskManagerServiceClient()
    Private _name As String

    Public Sub New(ByVal name As String)
        InitializeComponent()

        _name = name
    End Sub
    Private Sub TaskForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Me.Text = "Task Manager: " + _name

        GetTaskNumbersFromService()
    End Sub

    Private Sub _btnSubmitTask_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles _btnSubmitTask.Click

        Dim assignedTo As String = _txtAssignToName.Text
        Dim newTaskNum As Integer = _taskMgrProxy.AddTask(_rtbNewTaskDescription.Text, assignedTo)

        _toolStripStatusLabel.Text = String.Format("Task number {0} was submitted", newTaskNum)

        _rtbNewTaskDescription.Text = String.Empty
        _txtAssignToName.Text = String.Empty

        _rtbNewTaskDescription.Focus()

        GetTaskNumbersFromService()
    End Sub

    Private Sub GetTaskNumbersFromService()
        ' First clear the list...
        _taskList.Items.Clear()

        ' And then load it:
        Dim taskNums As Integer() = _taskMgrProxy.GetTasksByAssignedName(_name)

        For Each tn In taskNums
            _taskList.Items.Add(tn)
        Next
    End Sub

    Private Sub _taskList_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles _taskList.SelectedIndexChanged
        Dim taskNum As Integer = Integer.Parse(_taskList.SelectedItem.ToString())
        Dim taskDesc As String

        Try
            taskDesc = _taskMgrProxy.GetTaskDescription(taskNum)
        Catch ex As FaultException(Of FaultInfo)
            MessageBox.Show( _
            "There was a problem retrieving task description from the service.\n" _
                    + ex.Detail.Reason)
            taskDesc = String.Empty
        End Try

        _checkBoxCompleted.Checked = _taskMgrProxy.IsTaskCompleted(taskNum)
        _rtbTaskDescription.Text = taskDesc

    End Sub

    Private Sub _btnDeleteTask_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles _btnDeleteTask.Click
        Dim taskNum As Integer = Integer.Parse(_taskList.SelectedItem.ToString())

        Try
            _taskMgrProxy.DeleteTask(taskNum)

            GetTaskNumbersFromService()

            If _taskList.Items.Count > 0 Then
                _taskList.SelectedIndex = 0
            Else
                _rtbTaskDescription.Text = String.Empty
                _checkBoxCompleted.Checked = False
            End If
        Catch ex As FaultException(Of FaultInfo)
            MessageBox.Show( _
             "There was a problem deleting a task with the service.\n" + _
              ex.Detail.Reason + "\nPlease try again later.")
        End Try
    End Sub

    Private Sub _checkBoxCompleted_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles _checkBoxCompleted.CheckedChanged
        If _checkBoxCompleted.Checked Then
            Dim selIndex As Integer = _taskList.SelectedIndex
            Dim taskNum As Integer = Integer.Parse(_taskList.SelectedItem.ToString())

            Try
                _taskMgrProxy.MarkTaskCompleted(taskNum)
            Catch ex As FaultException(Of FaultInfo)
                MessageBox.Show( _
                 "There was a problem mark a task completed with the service.\n" + _
                  ex.Detail.Reason)
            End Try
        End If
    End Sub
End Class