﻿Imports System.ServiceModel
Imports TaskClient.TaskService.Proxy

Public Class TaskForm
    Private _taskMgrProxy As TaskManagerServiceClient = New TaskManagerServiceClient()
    Private _name As String

    Public Sub New(ByVal name As String)
        InitializeComponent()

        _name = name
    End Sub
    Private Sub TaskForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Me.Text = "Task Manager: " + _name

        GetTaskNumbersFromService()
    End Sub

    Private Sub _btnSubmitTask_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles _btnSubmitTask.Click
        Dim newTask As New Task()

        newTask.AssignedTo = _txtAssignToName.Text
        newTask.Description = _rtbNewTaskDescription.Text
        newTask.CreatedBy = _name
        newTask.DateCreated = DateTime.Now
        newTask.DateLastModified = DateTime.Now
        newTask.DueDate = _dateTimePicker.Value

        Dim taskAck As TaskAcknowledgement
        taskAck = _taskMgrProxy.AddTask(newTask)

        newTask.TaskState = taskAck.CurrentState
        Dim extraComment As String = String.Empty
        If taskAck.Comments <> Nothing Then
            extraComment = taskAck.Comments
        End If
        _toolStripStatusLabel.Text = String.Format( _
                "Task number {0} was submitted. {1}", _
                    newTask.TaskNumber, extraComment)

        _rtbNewTaskDescription.Text = String.Empty
        _txtAssignToName.Text = String.Empty

        _rtbNewTaskDescription.Focus()

        GetTaskNumbersFromService()
    End Sub

    Private Sub GetTaskNumbersFromService()
        ' First clear the list...
        _taskList.Items.Clear()

        _taskList.DisplayMember = "TaskNumber"

        ' And then load it:
        Dim tasks As Task() = _taskMgrProxy.GetTasksByAssignedName(_name)

        For Each t In tasks
            _taskList.Items.Add(t)
        Next
    End Sub

    Private Sub _taskList_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles _taskList.SelectedIndexChanged
        Dim selTask As Task
        selTask = _taskList.SelectedItem

        _checkBoxCompleted.Checked = (selTask.TaskState = TaskStates.CompletedByDueDate Or selTask.TaskState = TaskStates.CompletedPastDueDate)
        _rtbTaskDescription.Text = selTask.Description
    End Sub

    Private Sub _btnDeleteTask_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles _btnDeleteTask.Click
        Dim selTask As Task
        selTask = _taskList.SelectedItem

        Try
            _taskMgrProxy.DeleteTask(selTask.TaskNumber)

            GetTaskNumbersFromService()

            If _taskList.Items.Count > 0 Then
                _taskList.SelectedIndex = 0
            Else
                _rtbTaskDescription.Text = String.Empty
                _checkBoxCompleted.Checked = False
            End If
        Catch ex As FaultException(Of FaultInfo)
            MessageBox.Show( _
             "There was a problem deleting a task with the service.\n" + _
              ex.Detail.Reason + "\nPlease try again later.")
        End Try
    End Sub

    Private Sub _checkBoxCompleted_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles _checkBoxCompleted.CheckedChanged
        If _checkBoxCompleted.Checked Then
            Dim selTask As Task
            selTask = _taskList.SelectedItem

            If (selTask.TaskState = TaskStates.CompletedByDueDate Or _
                        selTask.TaskState = TaskStates.CompletedPastDueDate) Then
                Return
            End If

            Try
                _taskMgrProxy.MarkTaskCompleted(selTask.TaskNumber)

                ' Mark it locally as well:
                If DateTime.Now > selTask.DueDate Then
                    selTask.TaskState = TaskStates.CompletedPastDueDate
                Else
                    selTask.TaskState = TaskStates.CompletedByDueDate
                End If
            Catch ex As FaultException(Of FaultInfo)
                MessageBox.Show( _
                 "There was a problem mark a task completed with the service.\n" + _
                  ex.Detail.Reason)
            End Try
        End If
    End Sub
End Class