﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Transactions;
using System.Data.SqlClient;
using System.Configuration;

namespace TransactionalClient
{
    class Program
    {
        static void Main(string[] args)
        {
            AppDomain.CurrentDomain.SetData("DataDirectory",
                System.IO.Path.GetFullPath(System.IO.Path.Combine(AppDomain.CurrentDomain.BaseDirectory, @"..\..\..")));
            Console.WriteLine(String.Format("Before starting, the record count is {0}", getRecordCount()));

            using (TransactionalServiceProxy.UpdateServiceClient client = 
                new TransactionalClient.TransactionalServiceProxy.UpdateServiceClient())
            {
                using (TransactionScope ts = new TransactionScope(TransactionScopeOption.Required))
                {
                    int updatedRecords = 0;
                    updatedRecords += client.UpdateWithTransaction("INSERT INTO DemoTable VALUES (1, 'Demo Data')");
                    updatedRecords += client.UpdateWithTransaction("INSERT INTO DemoTable VALUES (2, 'More Data')");
                    Console.WriteLine(String.Format("The record count is {0}", updatedRecords));
                    ts.Dispose();
                }
                Console.ReadLine();
            }

            Console.WriteLine(String.Format("After the rollback, the record count is {0}", getRecordCount()));
            Console.ReadLine();
        }


        #region Private Methods

        private static int getRecordCount()
        {
            int result;

            string connectionString =
                ConfigurationManager.ConnectionStrings["TK503"].ConnectionString;
            using (SqlConnection cn = new SqlConnection(connectionString))
            {
                cn.Open();
                SqlCommand cmd = new SqlCommand("SELECT COUNT(*) FROM DemoTable", cn);
                result = Convert.ToInt32(cmd.ExecuteScalar());
            }
        return result;
        }

        #endregion
    }
}
