﻿Imports System.Threading

Public Class ThreadPoolSynchronizer
    Inherits SynchronizationContext
    Implements IDisposable

    Dim workItemQueue As Queue(Of WorkItem)
    Dim workerThreads() As WorkerThread
    Dim _itemAdded As Semaphore

    Public Sub New(ByVal poolSize As Integer)
        If poolSize <= 0 Then
            Throw New InvalidOperationException("Pool size cannot be zero")
        End If

        workItemQueue = New Queue(Of WorkItem)()

        ReDim workerThreads(poolSize)
        For index As Integer = 0 To poolSize - 1
            workerThreads(index) = New WorkerThread(index + 1, Me)
        Next
    End Sub


    Public Sub Close()
        For Each thread As WorkerThread In workerThreads
            If thread IsNot Nothing Then
                thread.Abort()
            End If
        Next
    End Sub

    Public Sub Abort()

        For Each thread As WorkerThread In workerThreads
            thread.Abort()
        Next
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Me.Close()
    End Sub

    Public Overrides Sub Post(ByVal method As SendOrPostCallback, ByVal state As Object)
        Dim _workItem As WorkItem = New WorkItem(method, state)
        QueueWorkItem(_workItem)
    End Sub

    Public Overrides Sub Send(ByVal method As SendOrPostCallback, ByVal state As Object)
        If SynchronizationContext.Current.Equals(Me) Then
            method(state)
            Return
        End If

        Dim _workItem As WorkItem = New WorkItem(method, state)
        QueueWorkItem(_workItem)
        _workItem.AsyncWaitHandle.WaitOne()
    End Sub

    Protected Property ItemAdded() As Semaphore
        Get
            If _itemAdded Is Nothing Then
                _itemAdded = New Semaphore(0, Int32.MaxValue)
            End If

            Return _itemAdded
        End Get

        Set(ByVal value As Semaphore)
            _itemAdded = value
        End Set
    End Property

    Friend Overridable Sub QueueWorkItem(ByVal _workItem As WorkItem)

        SyncLock (workItemQueue)
            workItemQueue.Enqueue(_workItem)
            ItemAdded.Release()
        End SyncLock

    End Sub

    Friend ReadOnly Property QueueEmpty() As Boolean
        Get
            SyncLock (workItemQueue)
                If workItemQueue.Count > 0 Then
                    Return False
                End If

                Return True

            End SyncLock
        End Get
    End Property

    Friend Overridable Function GetNext() As WorkItem

        ItemAdded.WaitOne(1000)
        SyncLock (workItemQueue)
            If workItemQueue.Count = 0 Then
                Return Nothing
            End If
            Return workItemQueue.Dequeue()
        End SyncLock

    End Function

End Class
